use std::{io::{Stdout, Write}, ops::BitOrAssign, time::Duration};

use axum::response::sse::Event;
use clap::builder::NonEmptyStringValueParser;
use lib::{client::Client, message::{Message, MessageKind}, message_read::MessageReader, player::Player};
use crossterm::{
    cursor, event::{self, poll, Event as TermEvent, KeyCode, KeyModifiers}, execute, queue, style, terminal::{
        disable_raw_mode, enable_raw_mode, Clear, ClearType, EnableLineWrap, EnterAlternateScreen
    }
};

const HELP: &str = r#"CIAOTERPIA
atm posso fare questo:
(ricorda di premere i bottoni)
- '1' mando giocante
- '2' mando lobbando (futuro)
"#;

fn create_message_player(player: &Player) -> anyhow::Result<Message, anyhow::Error> {
    Message::new(
        &player.get_name(),
        MessageKind::Test,
        player.encode()?
    )
}

async fn delete_player(player: &Player, client: reqwest::Client, addr: &str) -> anyhow::Result<()> {
    let player_id = &player.name;
    let response = client
        .get([&addr, "/delete/", &player_id].concat())
        .send().await;

    if let Err(e) = &response {
        println!("{e}\n");
        return Ok(())
    }

    println!("{}", response?.text().await?);

    Ok(())
}

async fn send_player(player: &Player, client: reqwest::Client, addr: &str) -> anyhow::Result<()> {
    let player_message = create_message_player(&player)?;
    let response = client
        .post([&addr, "/create/player"].concat())
        .header("Content-Type", "application/json")
        .body(player_message.encode()?)
        .send().await;

    if let Err(e) = &response {
        println!("{e}\n");
        return Ok(())
    }

    println!("{}", response?.text().await?);

    Ok(())
}
async fn handle_terminal(player: &Player, stdout: &mut Stdout, client: reqwest::Client, addr: &str) -> anyhow::Result<()> {
    'handler: loop {
        while let Ok(_event_happened) = poll(Duration::from_millis(2)) {
            match event::read().expect("diomerds") {
                TermEvent::Key(key) => {
                    match (key.code, key.modifiers) {
                        (KeyCode::Char('c'), KeyModifiers::CONTROL) => break 'handler,
                        (KeyCode::Char('h'), KeyModifiers::NONE) => {
                            for line in HELP.split('\n') {
                                println!("{line}")
                            }
                        }, 
                        (KeyCode::Char('1'), KeyModifiers::NONE) => {
                            send_player(&player, client.clone(), addr).await?;
                        },
                        (KeyCode::Char('2'), KeyModifiers::NONE) => {
                            delete_player(&player, client.clone(), addr).await?;
                        },
                        (_, _) => println!("{:?}\n", key),
                    }
                },
                _ => continue,
            }
        }
    }

    Ok(())
}

#[tokio::main]
pub async fn main() -> anyhow::Result<()> {
    let mut stdout = std::io::stdout();

    enable_raw_mode()?;
    execute!(
        stdout,
        Clear(ClearType::All),
        EnableLineWrap,
        cursor::Show,
        cursor::EnableBlinking,
        EnterAlternateScreen,
    )?;

    let player = Player::new()?;

    let addr = "http://127.0.0.1:8080";
    let client = reqwest::Client::new();

    // println!("{}", HELP);

    handle_terminal(&player, &mut stdout, client.clone(), &addr).await?;
    disable_raw_mode()?;

    Ok(())
}
